from office365api.model.model import Recipient
from office365api.model.model import EmailAddress
from office365api.model.model import Message
from office365api.model.model import ItemBody
from office365api.model.model import Attachment, FileAttachment, ItemAttachment



from office365api.connection import Connection
from office365api.model import Attachment
from office365api.model import Message


class Base(object):

    BASE_URL = 'https://outlook.office365.com/api/v1.0/me'
    SEND_URL = BASE_URL + '/sendmail'
    MAILBOX_URL = BASE_URL + '/folders/{folder_id}/messages'
    MESSAGE_URL = BASE_URL + '/messages/{id}'

    ATTACHMENT_URL = MESSAGE_URL + '/attachments?$expand=Microsoft.OutlookServices.ItemAttachment/Item'
    REPLY_URL = MESSAGE_URL+'/reply'
    REPLY_ALL_URL = MESSAGE_URL+'/replyall'
    FORWARD_URL = MESSAGE_URL+'/forward'

    def __init__(self, auth):
        self.auth = auth
        self.connection = Connection(auth)

    def get_messages_from_folder(self,
                                 folder,
                                 select=None,
                                 filters=None,
                                 search=None,
                                 order_by=None,
                                 top=100,
                                 skip=0):
        """
        Downloads messages to local memory.
        :param skip:  Page results, skip - default 0.
        :param top: Page size, default take first 50 messages.
        :param folder: The folder from where to get messages. [Inbox, Drafts, SentItems,
        DeletedItems]
        :param select: The list of additional fields to retrieve.
        ['Bcc', 'IsDeliveryReceiptRequested']. By default returns only fields required for
        Message class.
        :param filters: Filters for messages OData 4.0 compatible.
        Example: "From/EmailAddress/Address ne 'MicrosoftOffice365@email.office.com'"
        :param search: Search criteria. When supplying string looks in subject, body etc
        if you want to look in a particular field 'from:microsoft'
        :param order_by: Order by field name. Example: 'DateTimeReceived desc'
        """
        url = self.MAILBOX_URL.format(folder_id=folder)

        select = select or []
        select.extend(Message.parameters())
        params = {'$select': (','.join(select)), '$top': top, '$skip': skip}

        def add(k, v):
            if v:
                params[k] = v

        add('$search', search)
        add('$filter', filters)
        add('$orderby', order_by)

        # search override
        if search:
            for key in ['$skip', '$filter', '$orderby']:
                params.pop(key, None)

        response = self.connection.get(url=url, params=params)
        data = response.json()
        return [Message.from_dict(value) for value in data.get('value')] if data else []

    def get_attachments(self, message):
        """
        Lazy loaded Attachments.
        :param message: Message object.
        :return: Attachment collection. It is also added to message as side effect.
        """
        if not message.HasAttachments:
            return []
        response = self.connection.get(url=self.ATTACHMENT_URL.format(id=message.Id))
        data = response.json()
        message.Attachments = [Attachment.factory(a) for a in data.get('value', [])] \
            if data else []
        return message.Attachments

    def send_message(self, message):
        """
        Immediately sends the message.
        :param message: Message.
        :return: None
        """
        headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
        data = message.data
        self.connection.post(self.SEND_URL, json=data, headers=headers)

    def reply(self, message, comment=None, to_all=False):
        """
        Sends reply to sender and other recipients.
        :param message: Message to reply to, only Id is important.
        :param comment: Optional comment.
        :param to_all: If true reply to other recipients as well.
        :return: None
        """
        headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
        url = (self.REPLY_ALL_URL if to_all else self.REPLY_URL).format(id=message.Id)
        data = {'Comment': (comment or '')}
        self.connection.post(url=url, json=data, headers=headers)

    def forward(self, message, recipients, comment=None):
        """
        Sends reply to sender and other recipients.
        :param recipients: Recipients to forward it too.
        :param message: Message to reply to, only Id is important.
        :param comment: Optional comment.
        :return: None
        """
        headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
        url = self.FORWARD_URL.format(id=message.Id)
        data = {'Comment': (comment or ''), 'ToRecipients': [dict(r) for r in recipients]}
        self.connection.post(url=url, json=data, headers=headers)

    def delete_message(self, message):
        """
        Deletes message from the server.
        :param message: Message object.
        :return: None
        """
        self.delete_message_id(message_id=message.Id)

    def delete_message_id(self, message_id):
        """
        Deletes message from the server.
        :param message_id: Message id
        :return: None
        """
        url = self.MESSAGE_URL.format(id=message_id)
        self.connection.delete(url=url)

    def update_message(self, message, fields):
        """
        Deletes message from the server.
        :param fields: Fields needed updating.
        :param message: Message object.
        :return: None
        """
        url = self.MESSAGE_URL.format(id=message.Id)
        headers = {'Content-type': 'application/json', 'Accept': 'application/json'}
        self.connection.patch(url=url, data=fields, headers=headers)

    def create_attachment(self, message, attachment):
        """
        Adds an attachment to draft message before sending.
        :param message: The draft message.
        :param attachment: Attachment.
        :return: None
        """
        url = self.ATTACHMENT_URL.format(id=message.Id)
        self.connection.post(url=url, data=attachment.writable_properties)

    def delete_attachment(self, message, attachment):
        """
        Deletes attachment from message.
        :param message: The message.
        :param attachment: The attachment.
        :return:
        """
        url = self.ATTACHMENT_URL.format(id=message.Id) + '/' + attachment.Id
        self.connection.delete(url=url)

    def mark_read(self, message):
        """
        Marks messages read.
        :param message: Message to mark.
        :return:
        """
        read = {"IsRead": True}
        self.update_message(message=message, fields=read)



from requests import ConnectionError
import requests


class Connection(object):

    def __init__(self, auth):
        self.s = requests.Session()
        self.s.auth = auth

    def __del__(self):
        self.s.close()

    @staticmethod
    def validate_response(response):
        code = response.status_code
        if 200 > code or code > 299:
            raise ConnectionError("{code}: {text}".format(code=code, text=response.text))

    def get(self, url, params=None, **kwargs):
        response = self.s.get(url, params=params, **kwargs)
        self.validate_response(response)
        return response

    def post(self, url, data=None, json=None, **kwargs):
        response = self.s.post(url, data=data, json=json, **kwargs)
        self.validate_response(response)
        return response

    def delete(self, url, **kwargs):
        response = self.s.delete(url, **kwargs)
        self.validate_response(response)
        return response

    def patch(self, url, data, **kwargs):
        response = self.s.patch(url, data=data, **kwargs)
        self.validate_response(response)
        return response



from office365api.mail.base import Base
from office365api.mail.drafts import Drafts
from office365api.mail.inbox import Inbox


class Mail(Base):
    _inbox = None

    @property
    def inbox(self):
        if not self._inbox:
            self._inbox = Inbox(self.auth)
        return self._inbox

    _drafts = None

    @property
    def drafts(self):
        if not self._drafts:
            self._drafts = Drafts(self.auth)
        return self._drafts



#!/usr/bin/env python
from setuptools import setup
import sys
from os.path import normpath, dirname, join


def read_rst(f):
    with open('README.rst') as f:
        return f.read()

README = normpath(join(dirname(__file__), 'README.rst'))

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Office/Business :: Office Suites',
    'Topic :: Software Development :: Libraries'
]

setup(name='Office365Api-27',
      version= "0.0.7",
      description='Python library for working with Microsoft Office 365',
      long_description=read_rst(README),
      author='Dmitriy Krasnikov',
      author_email='dmitriy.krasnikov@swimlane.com',
      maintainer='Dmitriy Krasnikov',
      maintainer_email='dmitriy.krasnikov@swimlane.com',
      url='https://github.com/swimlane/python-office365',
      packages=['office365api', 'office365api/model', 'office365api/mail'],
      install_requires=['requests'],
      license='MIT',
      classifiers=CLASSIFIERS
      )



from office365api.mail.base import Base


class Folder(Base):
    @property
    def folder_name(self):
        raise NotImplementedError('This class cannot be used without inheritance.')

    def get_messages(self, select=None,
                     filters=None,
                     search=None,
                     order_by=None,
                     top=50,
                     skip=0):
        """
        Downloads messages to local memory.

        :param skip:  Page results, skip - default 0.

        :param top: Page size, default take first 50 messages.

        :param select: The list of additional fields to retrieve.
        ['Bcc', 'IsDeliveryReceiptRequested']. By default returns only fields required for
        Message class.

        :param filters: Filters for messages OData 4.0 compatible.
        Example: "From/EmailAddress/Address ne 'MicrosoftOffice365@email.office.com'"

        :param search: Search criteria. When supplying string looks in subject, body etc
        if you want to look in a particular field 'from:microsoft'

        :param order_by: Order by field name. Example: 'DateTimeReceived desc'
        """
        return self.get_messages_from_folder(folder=self.folder_name,
                                             select=select,
                                             filters=filters,
                                             search=search,
                                             order_by=order_by,
                                             top=top, skip=skip)
